#include <oiran/unused.hpp>
#include <oiran/rita/geometry.hpp>
#include "block.hpp"
#include "resource.hpp"


namespace {


#define CHECK_IS_ON_LINE_SEGMENT(position) \
	if(is_on_line_segment(position, ball.o, ball.p)) \
	{ \
		ball.push_orbit(ball.o); \
		ball.o = position; \
		ball.p = ball.o + (ball.p - ball.o) * -2; \
		return true; \
	}


	bool
	reflect(ball_t& ball,
			const vector_type& left_top, const vector_type& right_bottom)
	{
		vector_type left_bottom(left_top.x, right_bottom.y);
		vector_type right_top(right_bottom.x, left_top.y);

		if(ball.p == left_top || ball.p == left_bottom ||
				ball.p == right_top || ball.p == right_bottom)
		{
			ball.push_orbit(ball.o);
			ball.o = ball.p;
			ball.direction *= -1;
			return true;
		}

		CHECK_IS_ON_LINE_SEGMENT(left_top)
		CHECK_IS_ON_LINE_SEGMENT(left_bottom)
		CHECK_IS_ON_LINE_SEGMENT(right_top)
		CHECK_IS_ON_LINE_SEGMENT(right_bottom)

		if(ball.direction.x < 0)
		{
			if(collision_detection(ball.o, ball.p, right_top, right_bottom))
			{
				ball.push_orbit(ball.o);
				ball.o = intersection(ball.o, ball.p, right_top, right_bottom);
				ball.p.x -= (ball.p.x - ball.o.x) * 2;
				ball.direction.x *= -1;
				return true;
			}
			if(is_on_line_segment(ball.p, right_top, right_bottom))
			{
				ball.push_orbit(ball.o);
				ball.o = ball.p;
				ball.direction.x *= -1;
				return true;
			}
		}
		else if(ball.direction.x > 0)
		{
			if(collision_detection(ball.o, ball.p, left_top, left_bottom))
			{
				ball.push_orbit(ball.o);
				ball.o = intersection(ball.o, ball.p, left_top, left_bottom);
				ball.p.x -= (ball.p.x - ball.o.x) * 2;
				ball.direction.x *= -1;
				return true;
			}
			if(is_on_line_segment(ball.p, left_top, left_bottom))
			{
				ball.push_orbit(ball.o);
				ball.o = ball.p;
				ball.direction.x *= -1;
				return true;
			}
		}

		if(ball.direction.y < 0)
		{
			if(collision_detection(ball.o, ball.p, left_bottom, right_bottom))
			{
				ball.push_orbit(ball.o);
				ball.o = intersection(ball.o, ball.p, left_bottom, right_bottom);
				ball.p.y -= (ball.p.y - ball.o.y) * 2;
				ball.direction.y *= -1;
				return true;
			}
			if(is_on_line_segment(ball.p, left_bottom, right_bottom))
			{
				ball.push_orbit(ball.o);
				ball.o = ball.p;
				ball.direction.y *= -1;
				return true;
			}
		}
		else if(ball.direction.y > 0)
		{
			if(collision_detection(ball.o, ball.p, left_top, right_top))
			{
				ball.push_orbit(ball.o);
				ball.o = intersection(ball.o, ball.p, left_top, right_top);
				ball.p.y -= (ball.p.y - ball.o.y) * 2;
				ball.direction.y *= -1;
				return true;
			}
			if(is_on_line_segment(ball.p, left_top, right_top))
			{
				ball.push_orbit(ball.o);
				ball.o = ball.p;
				ball.direction.y *= -1;
				return true;
			}
		}
		return false;
	}


} // unnamed namespace.


block_t::block_t(const vector_type& left_top) :
		M_left_top(left_top),
		M_valid(true)
{
}

block_t::~block_t()
{
}


wall_t::wall_t(const vector_type& left_top) :
		block_t(left_top)
{
}

void
wall_t::draw(oiran::linda::graphics_t& g)
{
	oiran::unused(g);

	resource_t::instance()->images[wall - 1].draw(M_left_top.x, M_left_top.y);
}

bool
wall_t::collision_detection(ball_t& ball, bool& recheck)
{
	vector_type right_bottom(
			M_left_top.x + block_size, M_left_top.y + block_size);

	bool hit = reflect(ball, M_left_top, right_bottom);
	if(hit)
	{
		recheck = true;
		ball.speed += 0.01;
	}

	return hit;
}


normal_block_t::normal_block_t(const vector_type& left_top) :
		block_t(left_top)
{
}

void
normal_block_t::draw(oiran::linda::graphics_t& g)
{
	oiran::unused(g);

	resource_t::instance()->images[normal - 1].draw(M_left_top.x, M_left_top.y);
}

bool
normal_block_t::collision_detection(ball_t& ball, bool& recheck)
{
	vector_type right_bottom(
			M_left_top.x + block_size, M_left_top.y + block_size);

	bool hit = reflect(ball, M_left_top, right_bottom);
	if(hit)
	{
		recheck = true;
		ball.speed += 0.01;
		valid(false);
	}

	return hit;
}


block_t*
create_block(char id, const vector_type& left_top)
{
	switch(id)
	{
	case wall:		return new wall_t(left_top);
	case normal:	return new normal_block_t(left_top);
	}
	return 0;
}
