#include <SDL.h>
#include <exception>
#include <iostream>
#include <oiran/unused.hpp>
#include <oiran/linda/graphics.hpp>
#include <oiran/linda/module.hpp>
#include "defs.hpp"
#include "resource.hpp"
#include "scene.hpp"


int
main(int argc, char** argv)
{
	oiran::unused(argc);
	oiran::unused(argv);

	try {
		oiran::linda::module_t module;
		resource_t resource;

		// basic setting.
		module.title(name);
		module.framerate(framerate);
		module.screen(width, height);

		// resource setting.
		for(int i = 0; i < image_size; ++i)
			resource.images[i].load(image_files[i]);
		resource.font.load("res/font.dat");

		scene_manager_t scenes;
		scenes.push(scene_stage_list);

		oiran::linda::graphics_t g;
		g.clear_color(background_color);
		g.smooth();
		g.point_size(6);

		while(module.handle_event())
		{
			resource.mouse.update();
			scenes.update();

			g.clear();
			scenes.draw(g);
			g.flip();

			module.wait();
		}
		return 0;
	}

	catch(const std::exception& error)
	{ std::cerr << error.what() << std::endl; }
	catch(...)
	{ std::cerr << "unknown error." << std::endl; }

	return 1;
}
