#include <oiran/rita/geometry.hpp>
#include "defs.hpp"
#include "pad.hpp"
#include "resource.hpp"


pad_t::pad_t() :
		M_x(0)
{
}

void
pad_t::update()
{
	oiran::linda::mouse_t& mouse = resource_t::instance()->mouse;

	M_x = mouse.x() - game_offset_x;
	if(M_x < 0)
		M_x = 0;
	if(M_x >= field_width * block_size - pad_width)
		M_x = field_width * block_size - pad_width;
}

void
pad_t::draw(oiran::linda::graphics_t& g)
{
	g.color(0xffffff);
	g.line(M_x, pad_y, M_x + pad_width, pad_y);
}

bool
pad_t::collision_detection(ball_t& ball)
{
	if(ball.o == ball.p)
		return false;

	vector_type left(M_x, pad_y);
	vector_type right(M_x + pad_width, pad_y);

	bool hit = false;
	bool recheck = false;
	if(oiran::rita::collision_detection(ball.o, ball.p, left, right))
	{
		ball.push_orbit(ball.o);
		ball.o = intersection(ball.o, ball.p, left, right);
		ball.p.y = ball.o.y - (ball.p.y - ball.o.y);
		hit = true;
		recheck = true;
	}
	else if(is_on_line_segment(ball.p, left, right))
	{
		ball.push_orbit(ball.o);
		ball.o = ball.p;
		hit = true;
		recheck = false;
	}

	if(hit)
	{
		double x = ball.o.x;
		if(x < M_x + pad_width / 3)
		{
			double distance = M_x + pad_width / 3 - x;
			ball.direction.y = -1;
			ball.direction.x = -distance / (pad_width / 3) - 1;
			ball.direction = normalize(ball.direction);
		}
		else if(M_x + pad_width * 2 / 3 < x)
		{
			double distance = x - (M_x + pad_width * 2 / 3);
			ball.direction.y = -1;
			ball.direction.x = distance / (pad_width / 3) + 1;
			ball.direction = normalize(ball.direction);
		}
		else
		{
			ball.direction.y *= -1;
		}
		return recheck;
	}

	return false;
}
