#ifndef SCENE_HPP
#define SCENE_HPP

#include <oiran/noncopyable.hpp>
#include <oiran/linda/graphics.hpp>
#include <oiran/linda/scene.hpp>
#include "ball.hpp"
#include "defs.hpp"
#include "field.hpp"
#include "pad.hpp"


enum {
	scene_stage_list,
	scene_stage = scene_stage_list + max_stage,
	scene_edit = scene_stage + max_stage
};


class scene_manager_t;


class scene_stage_t : public oiran::linda::scene_t
{
public:
	enum {
		wait,
		on_play,
		game_clear,
		game_over
	};

	scene_stage_t(int stage);
	~scene_stage_t();

	void update();
	void draw(oiran::linda::graphics_t& g);

private:
	ball_t M_ball;
	field_t M_field;
	pad_t M_pad;

	int M_life;

	int M_state;
	int M_next_state;
	int M_effect_count;
};


class scene_edit_t : public oiran::linda::scene_t
{
public:
	scene_edit_t(int stage);

	void update();
	void draw(oiran::linda::graphics_t& g);

private:
	field_t M_field;

	char M_select;
	int M_x, M_y;

	int M_effect_count;
};


class scene_stage_list_t : public oiran::linda::scene_t
{
public:
	scene_stage_list_t(scene_manager_t* manager, int index);
	~scene_stage_list_t();

	void update();
	void draw(oiran::linda::graphics_t& g);

private:
	OIRAN_NONCOPYABLE(scene_stage_list_t);

	void _clear();
	void _reload();

private:
	scene_manager_t* M_manager;
	int M_index;
	int M_select;
	bool M_left;
	bool M_right;
	field_t* M_fields[9];
};


class scene_manager_t : public oiran::linda::scene_manager_t
{
protected:
	oiran::linda::scene_t* create(int id);
};


#endif // SCENE_HPP
