#include <oiran/random.hpp>
#include <oiran/linda/module.hpp>
#include "defs.hpp"
#include "scene.hpp"
#include "resource.hpp"


scene_stage_t::scene_stage_t(int stage) :
		M_field(stage),
		M_life(max_life),
		M_state(wait),
		M_next_state(wait),
		M_effect_count(0)
{
	oiran::linda::module_t::instance()->no_cursor();
	M_field.load();
}

scene_stage_t::~scene_stage_t()
{
	oiran::linda::module_t::instance()->cursor();
}

void
scene_stage_t::update()
{
	if(M_state != M_next_state)
		M_state = M_next_state;

	if(M_effect_count > 0)
		--M_effect_count;

	oiran::linda::mouse_t& mouse = resource_t::instance()->mouse;

	M_pad.update();

	switch(M_state)
	{
	case wait:
		if(mouse.is_pushed(mouse.left))
		{
			vector_type v(M_pad.x() + pad_width / 2, pad_y);
			M_ball.init(v);
			--M_life;
			M_next_state = on_play;
		}
		break;

	case on_play:
		M_ball.update();

		do {
			M_field.collision_detection(M_ball);
		} while(M_pad.collision_detection(M_ball));

		M_ball.push_orbit(M_ball.o);
		M_ball.push_orbit(M_ball.p);

		if(M_ball.p.x < 0 || field_width * block_size < M_ball.p.x ||
				M_ball.p.y < 0 || (field_height + 1) * block_size < M_ball.p.y ||
				(mouse.is_pressed(mouse.left) && mouse.is_pressed(mouse.right)))
		{
			if(M_life <= 0)
				M_next_state = game_over;
			else
				M_next_state = wait;
			M_effect_count = framerate / 5;
		}
		if(M_field.is_game_clear())
			M_next_state = game_clear;
		break;

	case game_over:
	case game_clear:
		if(mouse.is_pushed(mouse.left))
		{
			valid(false);
		}
		break;
	}
}

void
scene_stage_t::draw(oiran::linda::graphics_t& g)
{
	oiran::linda::simple_font_t& font = resource_t::instance()->font;

	g.push_matrix();
	if(M_effect_count)
		g.translate(game_offset_x + oiran::random(4) - 2,
				game_offset_y + oiran::random(4) - 2);
	else
		g.translate(game_offset_x, game_offset_y);

	g.color(0xffffff);
	M_field.draw(g);
	M_pad.draw(g);
	if(M_state == on_play)
		M_ball.draw(g);
	g.color(0xffffff);
	g.stroke();
	g.rect(0, 0, field_width * block_size, (field_height + 1) * block_size);
	g.fill();
	for(int i = 0; i < M_life; ++i)
		g.point((field_width + 1) * block_size + i * block_size, block_size / 2);

	g.color(0x888888);
	font.size(24, 24);
	if(M_state == game_over)
	{
		font.print(
				(field_width * block_size - 9 * 24) / 2,
				(field_height * block_size - 24) / 2,
				"GAME OVER");
	}
	if(M_state == game_clear)
	{
		font.print(
				(field_width * block_size - 10 * 24) / 2,
				(field_height * block_size - 24) / 2,
				"GAME CLEAR");
	}
	g.pop_matrix();
}
