#ifndef OIRAN_LINDA_JOYSTICK_HPP
#define OIRAN_LINDA_JOYSTICK_HPP

#include <SDL.h>
#include <iostream>
#include <oiran/noncopyable.hpp>

namespace oiran { namespace linda {


	// input device has some buttons.
	class button_input_t
	{
	public:
		enum {
			up,
			down,
			left,
			right,
			a_button,
			b_button,
			c_button,
			number_of_buttons
		};

		button_input_t();
		virtual ~button_input_t();

		virtual void update() = 0;

		int get(int id) const { return M_buttons[id]; }
		bool is_pressed(int id) const { return M_buttons[id] >= 1; }
		bool is_pushed(int id) const { return M_buttons[id] == 1; }
		bool is_released(int id) const { return M_buttons[id] <= 0; }
		bool is_pulled(int id) const { return M_buttons[id] == 0; }

	protected:
		void _update_buttons(unsigned int state);

	private:
		int M_buttons[number_of_buttons];
	};


	// button input state for replay.
	struct button_input_state_t
	{
		unsigned int state;
		unsigned int repeat;
	};


	// joystick.
	class joystick_t : public button_input_t
	{
	public:
		static const int axis_play;

		joystick_t();
		~joystick_t();

		void init();
		void update();

		void start_record(std::ostream& out);
		void stop_record();

	private:
		OIRAN_NONCOPYABLE(joystick_t);

	private:
		SDL_Joystick* M_stick;
		std::ostream* M_out;
		button_input_state_t M_replay_data;
	};


	// button input replay.
	class button_replay_t : public button_input_t
	{
	public:
		button_replay_t();

		void start_replay(std::istream& in);
		void stop_replay();

		void update();

		bool is_end() const { return !M_in; }

	private:
		OIRAN_NONCOPYABLE(button_replay_t);

	private:
		std::istream* M_in;
		button_input_state_t M_replay_data;
	};


}} // namespace oiran::linda.

#endif // OIRAN_LINDA_JOYSTICK_HPP
