#ifndef OIRAN_RITA_GEOMETRY_HPP
#define OIRAN_RITA_GEOMETRY_HPP

#include <cmath>
#include <oiran/rita/vector.hpp>

namespace oiran { namespace rita {


	// check the point on the line segment.
	template <typename T>
	inline bool
	is_on_line_segment(const vector_t<T, 2>& p,
			const vector_t<T, 2>& a, const vector_t<T, 2>& b)
	{
		return cross(b - a, p - a) == 0 &&
				dot(b - a, p - a) > 0 &&
				dot(a - b, p - b) > 0;
	}


	// collision detection. (line segment x line segment)
	template <typename T>
	inline bool
	collision_detection(
			const vector_t<T, 2>& a, const vector_t<T, 2>& b,
			const vector_t<T, 2>& c, const vector_t<T, 2>& d)
	{
		if(cross(b - a, c - a) * cross(b - a, d - a) < 0 &&
				cross(d - c, a - c) * cross(d - c, b - c) < 0)
			return true;
		return false;
	}


	// get intersection.
	template <typename T>
	inline vector_t<T, 2>
	intersection(
			const vector_t<T, 2>& a, const vector_t<T, 2>& b,
			const vector_t<T, 2>& c, const vector_t<T, 2>& d)
	{
		T d1 = std::abs(cross(d - c, a - c));
		T d2 = std::abs(cross(d - c, b - c));
		T t = d1 / (d1 + d2);
		return a + (b - a) * t;
	}


}} // namespace oiran::rita.

#endif // OIRAN_RITA_GEOMETRY_HPP
