#ifndef OIRAN_RITA_VECTOR_BASE_HPP
#define OIRAN_RITA_VECTOR_BASE_HPP

#include <cstddef>
#include <oiran/meta.hpp>

namespace oiran { namespace rita { namespace detail {


	// vector for 2D.
	template <typename T>
	class vector2_t
	{
	public:
		union {
			struct { T elems[2]; };
			struct { T x, y; };
		};

	protected:
		vector2_t() {}
		vector2_t(const T& a, const T& b) : x(a), y(b) {}
		explicit vector2_t(const T* a) : x(a[0]), y(a[1]) {}
		template <typename U>
		vector2_t(const vector2_t<U>& v) : x(v.x), y(v.y) {}

		template <typename U>
		void mul(const U& a)
		{ x *= a; y *= a; }

		template <typename U>
		void div(const U& a)
		{ x /= a; y /= a; }
	};


	// vector for 3D.
	template <typename T>
	class vector3_t
	{
	public:
		union {
			struct { T elems[3]; };
			struct { T x, y, z; };
		};

	protected:
		vector3_t() {}
		vector3_t(const T& a, const T& b, const T& c) : x(a), y(b), z(c) {}
		explicit vector3_t(const T* a) : x(a[0]), y(a[1]), z(a[2]) {}
		template <typename U>
		vector3_t(const vector3_t<U>& v) : x(v.x), y(v.y), z(v.z) {}

		template <typename U>
		void mul(const U& a)
		{ x *= a; y *= a; z *= a; }

		template <typename U>
		void div(const U& a)
		{ x /= a; y /= a; z /= a; }
	};


	template <typename T, std::size_t N>
	struct vector_base_t
	{
		typedef typename select_t<N == 2, vector2_t<T>,
				typename select_t<N == 3, vector3_t<T>,
				null_t
				>::type>::type type;
	};


}}} // namespace oiran::rita::detail.

#endif // OIRAN_RITA_VECTOR_BASE_HPP
