#ifndef OIRAN_SINGLETON_HPP
#define OIRAN_SINGLETON_HPP

#include <assert.h>
#include <oiran/noncopyable.hpp>

namespace oiran {


	// singleton pattern.
	template <class T>
	class singleton_t
	{
	public:
		// get a pointer to the instance.
		static T* instance()
		{
			assert(S_singleton_instance != 0 && "singleton is not yet.");
			return S_singleton_instance;
		}

	protected:
		singleton_t()
		{
			assert(S_singleton_instance == 0 && "singleton is already.");
			S_singleton_instance = static_cast<T*>(this);
		}

		~singleton_t()
		{
			S_singleton_instance = 0;
		}

	private:
		OIRAN_NONCOPYABLE(singleton_t);

	private:
		static T* S_singleton_instance;
	};

	template <class T>
	T* singleton_t<T>::S_singleton_instance = 0;


} // namespace oiran.

#endif // OIRAN_SINGLETON_HPP
